@@ -14,8 +14,7 @@ import (
 func TestListBuckets(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	// alphabetically unordered list of buckets
 	bucketPrefix := s3BucketFromTestName(t)
@@ -29,21 +28,24 @@ func TestListBuckets(t *testing.T) {
 
 	result.Assert(t, icmd.Success)
 
+	// find the first created bucket
+	startIdx := strings.Index(result.Stdout(), fmt.Sprintf("s3://%v-1", bucketPrefix))
+	got := result.Stdout()[startIdx:]
+
 	// expect ordered list
-	assertLines(t, result.Stdout(), map[int]compareFunc{
+	assertLines(t, got, map[int]compareFunc{
 		0: suffix("s3://%v-1", bucketPrefix),
 		1: suffix("s3://%v-2", bucketPrefix),
 		2: suffix("s3://%v-3", bucketPrefix),
 		3: suffix("s3://%v-4", bucketPrefix),
-	})
+	}, strictLineCheck(false))
 }
 
 // -json ls bucket
 func TestListBucketsJSON(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	// alphabetically unordered list of buckets
 	bucketPrefix := s3BucketFromTestName(t)
@@ -54,27 +56,31 @@ func TestListBucketsJSON(t *testing.T) {
 
 	cmd := s5cmd("--json", "ls")
 	result := icmd.RunCmd(cmd)
-
 	result.Assert(t, icmd.Success)
 
+	stdout := result.Stdout()
+	// find the first created bucket
+	startIdx := strings.Index(stdout, fmt.Sprintf(`"name":"%v-1"`, bucketPrefix))
+	// find the start of the line
+	startOfLine := strings.LastIndex(stdout[:startIdx], "{")
+	beginningOfOutput := stdout[startOfLine:]
+
 	// expect ordered list
-	assertLines(t, result.Stdout(), map[int]compareFunc{
+	assertLines(t, beginningOfOutput, map[int]compareFunc{
 		0: suffix(`"name":"%v-1"}`, bucketPrefix),
 		1: suffix(`"name":"%v-2"}`, bucketPrefix),
 		2: suffix(`"name":"%v-3"}`, bucketPrefix),
 		3: suffix(`"name":"%v-4"}`, bucketPrefix),
-	}, jsonCheck(true))
+	}, jsonCheck(true), strictLineCheck(false))
 }
 
 // ls bucket/object
 func TestListSingleS3Object(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	// create 2 files, expect 1.
@@ -95,11 +101,9 @@ func TestListSingleS3Object(t *testing.T) {
 func TestListSingleS3ObjectJSON(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	// create 2 files, expect 1.
@@ -120,11 +124,9 @@ func TestListSingleS3ObjectJSON(t *testing.T) {
 func TestListSingleWildcardS3Object(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 	putFile(t, s3client, bucket, "testfile1.txt", "this is a file content")
 	putFile(t, s3client, bucket, "testfile2.txt", "this is also a file content")
@@ -142,15 +144,39 @@ func TestListSingleWildcardS3Object(t *testing.T) {
 	}, alignment(true))
 }
 
-// ls -s bucket/object
-func TestListS3ObjectsWithDashS(t *testing.T) {
+func TestListWildcardS3ObjectWithNewLineInName(t *testing.T) {
 	t.Parallel()
 
 	bucket := s3BucketFromTestName(t)
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
+	createBucket(t, s3client, bucket)
+	putFile(t, s3client, bucket, "normal.txt", "this is a file content")
+	putFile(t, s3client, bucket, "another.txt", "this is another file content")
+	putFile(t, s3client, bucket, "newli\ne.txt", "this is yet another file content")
+	putFile(t, s3client, bucket, "nap.txt", "this, too, is a file content")
+
+	cmd := s5cmd("ls", "s3://"+bucket+"/n*.txt")
+	result := icmd.RunCmd(cmd)
+
+	result.Assert(t, icmd.Success)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: suffix("28 nap.txt"),
+		1: suffix("32 newli"),
+		2: equals("e.txt"),
+		3: suffix("22 normal.txt"),
+	})
+}
+
+// ls -s bucket/object
+func TestListS3ObjectsWithDashS(t *testing.T) {
+	t.Parallel()
+
+	s3client, s5cmd := setup(t)
+
+	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
 	putFile(t, s3client, bucket, "testfile1.txt", "this is a file content")
 
@@ -166,22 +192,20 @@ func TestListS3ObjectsWithDashS(t *testing.T) {
 func TestListMultipleWildcardS3Object(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
-	putFile(t, s3client, bucket, "/a/testfile1.txt", "content")
-	putFile(t, s3client, bucket, "/a/testfile2.txt", "content")
-	putFile(t, s3client, bucket, "/b/testfile3.txt", "content")
-	putFile(t, s3client, bucket, "/b/testfile4.txt", "content")
-	putFile(t, s3client, bucket, "/c/testfile5.gz", "content")
-	putFile(t, s3client, bucket, "/c/testfile6.txt.gz", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/file7.txt", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/testfile8.txt", "content")
-	putFile(t, s3client, bucket, "/e/txt/testfile9.txt.gz", "content")
-	putFile(t, s3client, bucket, "/f/txt/testfile10.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile1.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile2.txt", "content")
+	putFile(t, s3client, bucket, "b/testfile3.txt", "content")
+	putFile(t, s3client, bucket, "b/testfile4.txt", "content")
+	putFile(t, s3client, bucket, "c/testfile5.gz", "content")
+	putFile(t, s3client, bucket, "c/testfile6.txt.gz", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/file7.txt", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/testfile8.txt", "content")
+	putFile(t, s3client, bucket, "e/txt/testfile9.txt.gz", "content")
+	putFile(t, s3client, bucket, "f/txt/testfile10.txt", "content")
 
 	const pattern = "/*/testfile*.txt"
 	cmd := s5cmd("ls", "s3://"+bucket+pattern)
@@ -203,22 +227,20 @@ func TestListMultipleWildcardS3Object(t *testing.T) {
 func TestListMultipleWildcardS3ObjectWithPrefix(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
-	putFile(t, s3client, bucket, "/a/testfile1.txt", "content")
-	putFile(t, s3client, bucket, "/a/testfile2.txt", "content")
-	putFile(t, s3client, bucket, "/a/testfile3.txt", "content")
-	putFile(t, s3client, bucket, "/b/testfile4.txt", "content")
-	putFile(t, s3client, bucket, "/c/testfile5.gz", "content")
-	putFile(t, s3client, bucket, "/c/testfile6.txt.gz", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/file7.txt", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/testfile8.txt", "content")
-	putFile(t, s3client, bucket, "/e/txt/testfile9.txt.gz", "content")
-	putFile(t, s3client, bucket, "/f/txt/testfile10.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile1.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile2.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile3.txt", "content")
+	putFile(t, s3client, bucket, "b/testfile4.txt", "content")
+	putFile(t, s3client, bucket, "c/testfile5.gz", "content")
+	putFile(t, s3client, bucket, "c/testfile6.txt.gz", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/file7.txt", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/testfile8.txt", "content")
+	putFile(t, s3client, bucket, "e/txt/testfile9.txt.gz", "content")
+	putFile(t, s3client, bucket, "f/txt/testfile10.txt", "content")
 
 	const pattern = "/a/testfile*.txt"
 	cmd := s5cmd("ls", "s3://"+bucket+pattern)
@@ -237,22 +259,20 @@ func TestListMultipleWildcardS3ObjectWithPrefix(t *testing.T) {
 func TestListS3ObjectsAndFolders(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 	putFile(t, s3client, bucket, "testfile1.txt", "content")
 	putFile(t, s3client, bucket, "report.gz", "content")
-	putFile(t, s3client, bucket, "/a/testfile2.txt", "content")
-	putFile(t, s3client, bucket, "/b/testfile3.txt", "content")
-	putFile(t, s3client, bucket, "/b/testfile4.txt", "content")
-	putFile(t, s3client, bucket, "/c/testfile5.gz", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/file7.txt", "content")
-	putFile(t, s3client, bucket, "/d/foo/bar/testfile8.txt", "content")
-	putFile(t, s3client, bucket, "/e/txt/testfile9.txt.gz", "content")
-	putFile(t, s3client, bucket, "/f/txt/testfile10.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile2.txt", "content")
+	putFile(t, s3client, bucket, "b/testfile3.txt", "content")
+	putFile(t, s3client, bucket, "b/testfile4.txt", "content")
+	putFile(t, s3client, bucket, "c/testfile5.gz", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/file7.txt", "content")
+	putFile(t, s3client, bucket, "d/foo/bar/testfile8.txt", "content")
+	putFile(t, s3client, bucket, "e/txt/testfile9.txt.gz", "content")
+	putFile(t, s3client, bucket, "f/txt/testfile10.txt", "content")
 
 	cmd := s5cmd("ls", "s3://"+bucket)
 	result := icmd.RunCmd(cmd)
@@ -275,16 +295,14 @@ func TestListS3ObjectsAndFolders(t *testing.T) {
 func TestListS3ObjectsAndFoldersWithPrefix(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 	putFile(t, s3client, bucket, "testfile1.txt", "content")
 	putFile(t, s3client, bucket, "report.gz", "content")
-	putFile(t, s3client, bucket, "/a/testfile2.txt", "content")
-	putFile(t, s3client, bucket, "/t/testfile3.txt", "content")
+	putFile(t, s3client, bucket, "a/testfile2.txt", "content")
+	putFile(t, s3client, bucket, "t/testfile3.txt", "content")
 
 	// search with prefix t
 	cmd := s5cmd("ls", "s3://"+bucket+"/t")
@@ -302,11 +320,9 @@ func TestListS3ObjectsAndFoldersWithPrefix(t *testing.T) {
 func TestListNonexistingS3ObjectInGivenPrefix(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	const pattern = "/*/testfile*.txt"
@@ -318,7 +334,7 @@ func TestListNonexistingS3ObjectInGivenPrefix(t *testing.T) {
 	assertLines(t, result.Stdout(), map[int]compareFunc{})
 
 	assertLines(t, result.Stderr(), map[int]compareFunc{
-		0: equals(`ERROR "ls s3://test-list-nonexisting-s-3-object-in-given-prefix/*/testfile*.txt": no object found`),
+		0: equals(`ERROR "ls s3://%v/*/testfile*.txt": no object found`, bucket),
 	}, strictLineCheck(false))
 }
 
@@ -326,11 +342,9 @@ func TestListNonexistingS3ObjectInGivenPrefix(t *testing.T) {
 func TestListNonexistingS3Object(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	cmd := s5cmd("ls", "s3://"+bucket+"/nosuchobject")
@@ -349,11 +363,9 @@ func TestListNonexistingS3Object(t *testing.T) {
 func TestListS3ObjectsWithDashE(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	putFile(t, s3client, bucket, "testfile1.txt", strings.Repeat("this is a file content", 10000))
@@ -374,11 +386,9 @@ func TestListS3ObjectsWithDashE(t *testing.T) {
 func TestListS3ObjectsWithDashH(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	putFile(t, s3client, bucket, "testfile1.txt", strings.Repeat("this is a file content", 10000))
@@ -415,10 +425,11 @@ func TestListS3ObjectsWithExcludeFilter(t *testing.T) {
 		"a/file.c",
 		"file2.txt",
 		"file2.txt.extension", // this should not be excluded.
+		"newli\ne",
+		"newli\ne.txt",
 	}
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	createBucket(t, s3client, bucket)
 
@@ -436,7 +447,9 @@ func TestListS3ObjectsWithExcludeFilter(t *testing.T) {
 		1: match(`a/try.py`),
 		2: match(`file.py`),
 		3: match(`file2.txt.extension`),
-	}, trimMatch(dateRe), alignment(true))
+		4: match("newli"),
+		5: match("e"),
+	}, trimMatch(dateRe), alignment(false))
 }
 
 // ls --exclude ".txt" --exclude ".py" s3://bucket
@@ -461,8 +474,7 @@ func TestListS3ObjectsWithExcludeFilters(t *testing.T) {
 		"file2.txt",
 	}
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	createBucket(t, s3client, bucket)
 
@@ -497,8 +509,7 @@ func TestListS3ObjectsWithEmptyExcludeFilter(t *testing.T) {
 		"file2.txt",
 	}
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	createBucket(t, s3client, bucket)
 
@@ -534,8 +545,7 @@ func TestListNestedPrefixedS3ObjectsWithExcludeFilter(t *testing.T) {
 		"some-dir/some-dir/some-dir/file.txt",
 	}
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	createBucket(t, s3client, bucket)
 
@@ -585,8 +595,7 @@ func TestListLocalFilesWithExcludeFilter(t *testing.T) {
 		t.Run(tc.name, func(t *testing.T) {
 			t.Parallel()
 
-			_, s5cmd, cleanup := setup(t)
-			defer cleanup()
+			_, s5cmd := setup(t)
 
 			const excludePattern = "main*"
 			folderLayout := []fs.PathOp{
@@ -625,8 +634,7 @@ func TestListLocalFilesWithExcludeFilter(t *testing.T) {
 func TestListLocalFilesWithExcludeFilters(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	const (
 		excludePattern1 = "main*"
@@ -666,8 +674,7 @@ func TestListLocalFilesWithExcludeFilters(t *testing.T) {
 func TestListLocalFilesWithPrefixAndExcludeFilter(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	const (
 		excludePattern1 = "main*"
@@ -706,8 +713,7 @@ func TestListLocalFilesWithPrefixAndExcludeFilter(t *testing.T) {
 func TestListNestedLocalFolders(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	const (
 		excludePattern1 = "some-dir/some-dir/file.txt"
