@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"path/filepath"
 	"runtime"
+	"strings"
 	"testing"
 
 	"gotest.tools/v3/assert"
@@ -15,11 +16,9 @@ import (
 func TestRemoveSingleS3Object(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	const (
@@ -49,11 +48,9 @@ func TestRemoveSingleS3Object(t *testing.T) {
 func TestRemoveSingleS3ObjectJSON(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	const (
@@ -89,11 +86,9 @@ func TestRemoveSingleS3ObjectJSON(t *testing.T) {
 func TestRemoveMultipleS3Objects(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -132,11 +127,9 @@ func TestRemoveMultipleS3Objects(t *testing.T) {
 func TestRemoveMultipleS3ObjectsJSON(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -203,13 +196,11 @@ func TestRemoveTenThousandS3Objects(t *testing.T) {
 	// flaky test, skip it
 	t.Skip()
 
-	bucket := s3BucketFromTestName(t)
-
 	// ten thousand s3 objects are created for this test. by default, s3 backend is
 	// bolt but we need speed for this test, hence use in-memory storage.
-	s3client, s5cmd, cleanup := setup(t, withS3Backend("mem"))
-	defer cleanup()
+	s3client, s5cmd := setup(t, withS3Backend("mem"))
 
+	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
 
 	const filecount = 10000
@@ -251,11 +242,14 @@ func TestRemoveTenThousandS3Objects(t *testing.T) {
 func TestRemoveS3PrefixWithoutSlash(t *testing.T) {
 	t.Parallel()
 
+	// GCS throws key does not exist error, if an object can't be found
+	// as it does not use multi-delete. This behavior is different from
+	// Amazon S3, thus skip this test when used with GCS.
+	skipTestIfGCS(t, "GCS throws key does not exist error, which is a different output then aws")
+
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	const prefix = "prefix"
@@ -277,8 +271,7 @@ func TestRemoveS3PrefixWithoutSlash(t *testing.T) {
 func TestRemoveSingleLocalFile(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	const (
 		filename = "testfile1.txt"
@@ -308,8 +301,7 @@ func TestRemoveSingleLocalFile(t *testing.T) {
 func TestRemoveMultipleLocalFilesShouldNotFail(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	filesToContent := map[string]string{
 		"testfile1.txt":          "this is a test file 1",
@@ -352,8 +344,7 @@ func TestRemoveMultipleLocalFilesShouldNotFail(t *testing.T) {
 func TestRemoveLocalDirectory(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	folderLayout := []fs.PathOp{
 		fs.WithDir(
@@ -393,8 +384,7 @@ func TestRemoveLocalDirectoryWithGlob(t *testing.T) {
 		t.Skip("Files in Windows cannot contain glob(*) characters")
 	}
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	folderLayout := []fs.PathOp{
 		fs.WithDir(
@@ -445,8 +435,7 @@ func TestRemoveLocalDirectoryWithGlob(t *testing.T) {
 func TestVariadicMultipleLocalFilesWithDirectory(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	folderLayout := []fs.PathOp{
 		fs.WithDir(
@@ -482,8 +471,12 @@ func TestVariadicMultipleLocalFilesWithDirectory(t *testing.T) {
 func TestVariadicRemoveS3Objects(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	// GCS throws key does not exist error, if an object can't be found
+	// as it does not use multi-delete. This behavior is different from
+	// Amazon S3, thus skip this test when used with GCS.
+	skipTestIfGCS(t, "GCS throws key does not exist error, which is a different output then aws")
+
+	s3client, s5cmd := setup(t)
 
 	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
@@ -532,8 +525,7 @@ func TestVariadicRemoveS3Objects(t *testing.T) {
 func TestVariadicRemoveS3ObjectsWithWildcard(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
@@ -583,10 +575,9 @@ func TestVariadicRemoveS3ObjectsWithWildcard(t *testing.T) {
 func TestRemoveMultipleMixedObjects(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
-	const bucket = "bucket"
+	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
 
 	const (
@@ -624,11 +615,9 @@ func TestRemoveMultipleMixedObjects(t *testing.T) {
 func TestRemoveMultipleS3ObjectsDryRun(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -665,11 +654,9 @@ func TestRemoveMultipleS3ObjectsDryRun(t *testing.T) {
 func TestRemoveS3ObjectRawFlag(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -719,11 +706,9 @@ func TestRemoveS3ObjectRawFlag(t *testing.T) {
 func TestRemoveS3ObjectsPrefixRawFlag(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -765,11 +750,14 @@ func TestRemoveS3ObjectsPrefixRawFlag(t *testing.T) {
 func TestRemoveS3PrefixRawFlag(t *testing.T) {
 	t.Parallel()
 
+	// GCS gives key does not exist error, if an object can't be found
+	// as it does not use multi-delete. This behavior is different from
+	// Amazon S3, thus skip this test when used with GCS.
+	skipTestIfGCS(t, "GCS throws key does not exist error, which is a different output then aws")
+
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	filesToContent := map[string]string{
@@ -805,11 +793,9 @@ func TestRemoveS3PrefixRawFlag(t *testing.T) {
 func TestRemoveMultipleS3ObjectsWithExcludeFilter(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 	const excludePattern = "*.txt"
 
@@ -869,11 +855,9 @@ func TestRemoveMultipleS3ObjectsWithExcludeFilter(t *testing.T) {
 func TestRemoveMultipleS3ObjectsWithExcludeFilters(t *testing.T) {
 	t.Parallel()
 
+	s3client, s5cmd := setup(t)
+
 	bucket := s3BucketFromTestName(t)
-
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
-
 	createBucket(t, s3client, bucket)
 
 	const (
@@ -936,8 +920,7 @@ func TestRemoveMultipleS3ObjectsWithExcludeFilters(t *testing.T) {
 func TestRemoveS3ObjectsWithEmptyExcludeFilter(t *testing.T) {
 	t.Parallel()
 
-	s3client, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	s3client, s5cmd := setup(t)
 
 	bucket := s3BucketFromTestName(t)
 	createBucket(t, s3client, bucket)
@@ -1009,8 +992,7 @@ func TestRemoveLocalDirectoryWithExcludeFilter(t *testing.T) {
 		t.Run(tc.name, func(t *testing.T) {
 			t.Parallel()
 
-			_, s5cmd, cleanup := setup(t)
-			defer cleanup()
+			_, s5cmd := setup(t)
 
 			folderLayout := []fs.PathOp{
 				fs.WithDir(
@@ -1059,8 +1041,7 @@ func TestRemoveLocalDirectoryWithExcludeFilter(t *testing.T) {
 func TestRemoveLocalFilesWithExcludeFilters(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	folderLayout := []fs.PathOp{
 		fs.WithDir(
@@ -1116,8 +1097,7 @@ func TestRemoveLocalFilesWithExcludeFilters(t *testing.T) {
 func TestRemoveLocalFilesWithPrefixandExcludeFilters(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	folderLayout := []fs.PathOp{
 		fs.WithDir(
@@ -1175,8 +1155,7 @@ func TestRemoveLocalFilesWithPrefixandExcludeFilters(t *testing.T) {
 func TestRemovetNonexistingLocalFile(t *testing.T) {
 	t.Parallel()
 
-	_, s5cmd, cleanup := setup(t)
-	defer cleanup()
+	_, s5cmd := setup(t)
 
 	cmd := s5cmd("rm", "nonexistentfile")
 	result := icmd.RunCmd(cmd)
@@ -1189,3 +1168,136 @@ func TestRemovetNonexistingLocalFile(t *testing.T) {
 		0: equals(`ERROR "rm nonexistentfile": no object found`),
 	}, strictLineCheck(true))
 }
+
+func TestVersionedListAndRemove(t *testing.T) {
+	skipTestIfGCS(t, "versioning is not supported in GCS")
+
+	t.Parallel()
+
+	bucket := s3BucketFromTestName(t)
+
+	// versioninng is only supported with in memory backend!
+	s3client, s5cmd := setup(t, withS3Backend("mem"))
+
+	const (
+		filename              = "testfile.txt"
+		firstContent          = "this is the first content"
+		firstExpectedContent  = firstContent + "\n"
+		secondContent         = "this is the second content: different than the first."
+		secondExpectedContent = secondContent + "\n"
+	)
+
+	workdir := fs.NewDir(t, t.Name(), fs.WithFile(filename+"1", firstContent), fs.WithFile(filename+"2", firstContent))
+	defer workdir.Remove()
+
+	// create a bucket and Enable versioning
+	createBucket(t, s3client, bucket)
+	setBucketVersioning(t, s3client, bucket, "Enabled")
+
+	// upload two versions of the file with same key
+	putFile(t, s3client, bucket, filename, firstExpectedContent)
+	putFile(t, s3client, bucket, filename, secondExpectedContent)
+
+	//  remove (add a delete marker)
+	cmd := s5cmd("rm", "s3://"+bucket+"/"+filename)
+	result := icmd.RunCmd(cmd)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: contains("rm s3://%v/%v", bucket, filename),
+	})
+
+	// we expect to see it empty
+	cmd = s5cmd("ls", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+
+	assert.Assert(t, result.Stdout() == "")
+
+	// we expect to see 3 versions of objects one of which is a delete marker
+	cmd = s5cmd("ls", "--all-versions", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: contains("%v", filename),
+		1: contains("%v", filename),
+		2: contains("%v", filename),
+	})
+
+	// we expect all 3 versions of objects (one of which is a delete marker) to be deleted
+	cmd = s5cmd("rm", "--all-versions", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: contains("rm s3://%v/%v", bucket, filename),
+		1: contains("rm s3://%v/%v", bucket, filename),
+		2: contains("rm s3://%v/%v", bucket, filename),
+	})
+
+	// all versions are deleted so we don't expect to see any result
+	cmd = s5cmd("ls", "--all-versions", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+	assert.Assert(t, result.Stdout() == "")
+}
+
+func TestRemoveByVersionID(t *testing.T) {
+	skipTestIfGCS(t, "versioning is not supported in GCS")
+
+	t.Parallel()
+
+	bucket := s3BucketFromTestName(t)
+
+	// versioninng is only supported with in memory backend!
+	s3client, s5cmd := setup(t, withS3Backend("mem"))
+
+	const filename = "testfile.txt"
+
+	var contents = []string{
+		"This is first content",
+		"Second content it is, and it is a bit longer!!!",
+	}
+
+	// create a bucket and Enable versioning
+	createBucket(t, s3client, bucket)
+	setBucketVersioning(t, s3client, bucket, "Enabled")
+
+	// upload two versions of the file with same key
+	putFile(t, s3client, bucket, filename, contents[0])
+	putFile(t, s3client, bucket, filename, contents[1])
+
+	//  remove (add a delete marker)
+	cmd := s5cmd("rm", "s3://"+bucket+"/"+filename)
+	result := icmd.RunCmd(cmd)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: contains("rm s3://%v/%v", bucket, filename),
+	})
+
+	// we expect to see 3 versions of objects
+	cmd = s5cmd("ls", "--all-versions", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+
+	assertLines(t, result.Stdout(), map[int]compareFunc{
+		0: contains("%v", filename),
+		1: contains("%v", filename),
+		2: contains("%v", filename),
+	})
+
+	// now we will parse their version IDs and remove them one by one
+	versionIDs := make([]string, 0)
+	for _, row := range strings.Split(result.Stdout(), "\n") {
+		if row != "" {
+			arr := strings.Split(row, " ")
+			versionIDs = append(versionIDs, arr[len(arr)-1])
+		}
+	}
+
+	for _, version := range versionIDs {
+		cmd = s5cmd("rm", "--version-id", version,
+			fmt.Sprintf("s3://%v/%v", bucket, filename))
+		_ = icmd.RunCmd(cmd)
+	}
+
+	// all versions are deleted so we don't expect to see any result
+	cmd = s5cmd("ls", "--all-versions", "s3://"+bucket+"/"+filename)
+	result = icmd.RunCmd(cmd)
+	assert.Assert(t, result.Stdout() == "")
+}
